unit RogerSpecialPanel;

interface

uses Windows, SysUtils, Messages, Classes, Controls, Graphics, StdCtrls, ExtCtrls,
Forms, Dialogs, DsgnIntf;

type

  TDisplay = (diSimple, diAutosize, diTile, diScretch,diKeepAscpect, diCenter);
  TRogerSpecialPanel = class(TCustomPanel)
  private
    fOnPanelMoved:TNotifyEvent;
    fmoveable:boolean;
    FSelecionado:boolean;
    fmarge:integer;
    FBitmap: TBitmap;
    FDisplay: TDisplay;
    FOnDisplayChange: TNotifyEvent;
    FOnSelectClick: TNotifyEvent;
    procedure BitmapChanged(Sender: TObject);
    procedure SetSelecionado(Value: Boolean);
    procedure SetBitmap(Value: TBitmap);
    procedure SetDisplay(Value: TDisplay);
    procedure Paint; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WMEraseBkgnd(var m: TWMEraseBkgnd);
                           message WM_ERASEBKGND;
  protected
    procedure PanelMoved(var M: Tmessage); message WM_MOVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  public
    property canvas;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Selected: Boolean read FSelecionado Write SetSelecionado default false;
    property Ctl3D;
    property BorderWidth;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
  published
    property OnPanelMove:TNotifyEvent read fOnPanelMoved write fOnPanelMoved;
    Property Moveable:boolean read fmoveable write fmoveable default false;
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property Display: TDisplay read FDisplay write SetDisplay default diAutoSize;
    property OnDisplayChange: TNotifyEvent read FOnDisplayChange write FOnDisplayChange;
    property OnSelectClick: TNotifyEvent read FOnSelectClick write FOnSelectClick;
    property Align;
    property BorderStyle;
    property Enabled;
    property Color;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDrag;
  end;

  procedure Register;

implementation

{ TRogerSpecialPanel }
constructor TRogerSpecialPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fmarge:=6;
  FDisplay:=diAutoSize;
  FBitmap := TBitmap.Create;
  FBitmap.OnChange := BitmapChanged;
  FSelecionado:=false;
  fmoveable:=false;
end;

procedure TRogerSpecialPanel.CreateParams(var Params: TCreateParams);
begin
    BevelInner:=bvNone;
    BevelOuter:=bvNone;
    borderwidth:=1;
    Ctl3D:=false;
    caption:='';
    inherited;
end;

procedure TRogerSpecialPanel.SetSelecionado(Value: Boolean);
begin
     if FSelecionado=Value then exit;
     FSelecionado:=Value;
     if FSelecionado then
     begin
          canvas.Pen.Color:=clRed;
          canvas.Pen.Style:= psDot;
          canvas.Pen.Width:=1;
          Canvas.Rectangle(1,1,width-3,height-3);
     end
     else
         repaint;
end;

destructor TRogerSpecialPanel.Destroy;
begin
  FBitmap.Free;
  inherited Destroy;
end;

procedure TRogerSpecialPanel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
     inherited;
     if assigned(FonSelectClick) then FonSelectClick(self);
     SetSelecionado(true);
end;

procedure TRogerSpecialPanel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
     inherited;
     SetSelecionado(false);
end;

procedure TRogerSpecialPanel.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
     inherited;
     if FSelecionado and fmoveable and (ssleft in shift) then
     begin
          releasecapture;
          perform(wm_nclbuttondown,htcaption,0);
     end
     else
         SetSelecionado(false);
end;

procedure TRogerSpecialPanel.PanelMoved(var M: Tmessage);
begin
  if assigned(FonPanelMoved) then FOnPanelMoved(self);
  inherited;
end;


procedure TRogerSpecialPanel.BitmapChanged(Sender: TObject);
begin
  if HandleAllocated then
    RedrawWindow(Handle, nil, 0, RDW_INVALIDATE or
      RDW_ERASE or RDW_ALLCHILDREN);
end;

procedure TRogerSpecialPanel.SetBitmap(Value: TBitmap);
begin
  FBitmap.Assign(Value);
end;

procedure TRogerSpecialPanel.SetDisplay(Value: TDisplay);
begin
  if FDisplay <> value then
  begin
     FDisplay:=value;
     SetBitmap(FBitmap);
     if assigned(FonDisplayChange) then FOnDisplayChange(self);
  end;
end;

procedure TRogerSpecialPanel.WMEraseBkgnd(var m : TWMEraseBkgnd);
begin
  m.Result := LRESULT(False);
end;

procedure TRogerSpecialPanel.Paint;
var
  x, y,z,w: Integer;
  p: real;
begin
  inherited paint;
  if not(FBitmap.Empty) then
begin
// ------- so desenha em 0,0
if FDisplay =diSimple then
  begin
    if FBitmap.Width > width then Width := FBitmap.Width;
    if FBitmap.height > height then height := FBitmap.height;
    Canvas.Draw(0, 0, FBitmap);
  end;
// ------- Autosize
if FDisplay=diAutoSize then
   if align <> alNone then
      FDisplay:=diCenter
   else
      begin
           Width := FBitmap.Width;
           Height := FBitmap.Height;
           Canvas.Draw(0, 0, FBitmap);
      end;
// ------- Lado a Lado
if FDisplay=diTile then
   begin
    z := FBitmap.Height;
    w := FBitmap.Width;
    y := 0;
    while y < Height do begin
      x := 0;
      while x < Width do begin
        Canvas.Draw(x, y, FBitmap);
        x := x + w;
      end;
      y := y + z;
    end;
    end;
// --------- Scretch
if FDisplay=diScretch then
  begin
    Canvas.StretchDraw(rect(0, 0,width,height), FBitmap);
  end;
// --------- center
if FDisplay=diCenter then
  begin
    if FBitmap.Width <= width  then x := trunc((width-FBitmap.Width)/2) else x:=0;
    if FBitmap.height<= height then y := trunc((height-FBitmap.height)/2) else y:=0;
    Canvas.Draw(x, y, FBitmap);
  end;
// --------- KeepAspect
if FDisplay=diKeepAscpect then
   begin
   p:= FBitmap.height/FBitmap.width;
   y:= Width;
   x:=trunc(y*p);
   if height < x then
      begin
        x:=height;
        y:=trunc(x/p);
      end;
   if y < width  then w:=trunc((width-y)/2) else w:=0;
   if x < height then z:=trunc((height-x)/2) else z:=0;
   Canvas.StretchDraw(rect(w,z,w+y,z+x), FBitmap);
   end;
end;
end;

procedure Register;
begin
 RegisterComponents('RogerLib2', [TRogerSpecialPanel]);
end;

end.
